#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gnomeagenda.pluginsystem import *
import cgi
import re, htmlentitydefs

class CalendarProviderBase(Plugin):
	capabilities = []
	__gsignals__ = {
		#'updating': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,[gobject.TYPE_FLOAT]),
	    'event-added': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,[]),
	    'calendar-added': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,[gobject.TYPE_STRING]),
	    'notify-progress' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,[]),
	    'notify-query-provider' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,[]),
	    'notify-query-calendar': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,[gobject.TYPE_STRING]),
	}
	"""This is the base class for all calendar Providers"""
	def __init__(self):
		Plugin.__init__(self)
		self.errors = []
		self.timezoneadjust = 0
		self.disabled = False
		self.eventStorage = []
		self.calendarStorage = {}
		self.update_ok = False
	def initPlugin(self,options):
		self.setOptions(options)
		pass
	def notify_query_provider(self):
		self.emit('notify-query-provider')
		self.notify()
	def notify_query_calendar(self,name):
		#print "calendar %s is checked" % name
		self.emit('notify-query-calendar',name)
		self.notify()		
	def clearErrors(self):
		self.errors = []
	def notifyError(self,errorMessage):
		"""Notify the user on Errors that occured during the update"""
		print errorMessage
		self.errors.append(errorMessage)
	def getErrors(self):
		"""return the errors"""
		return self.errors
	def deleteEvents(self):
		self.eventStorage = []
		self.calendarStorage = {}		
	def addEvent(self, id, url, start, end, title, description = "", place = "", status = "CONFIRMED"):
		"""add a new event to the internal even storage."""
		title = self.ensureString(title)
		description = self.ensureString(description)
		place = self.ensureString(place)

		title = self.escape(self.unescape(title))
		description = self.escape(self.unescape(description))
		place = self.escape(self.unescape(place))
		#print (id, url, start, end, title, description, place, status)
		self.eventStorage.append({'id':id,'cal_id':url,'start':start,'end':end,'title':title,'description': description,'place':place,'status':status})
		
		self.emit('event-added')
		self.notify()
	def notify(self):
		self.emit('notify-progress')
	def addCalendar(self,id,name,color,updated_ext = None,hidden = False):
		"""
		adds a new calendar to the local storage
		emits the 'calendar-added' signal
		"""
		name = self.escape(self.unescape(name))		
		if(not self.calendarStorage.has_key(id)):
			self.calendarStorage[id] = [id,name,color,updated_ext,hidden]
			#print "added Calendar %s" % name
			#print (id,name,color,updated_ext,hidden)
			self.emit('calendar-added',name)
			self.notify()
	def ensureString(self,val):
		if (val == None):
			return ""
		else:
			return val
	def unescape(self,text):
		def fixup(m):
			text = m.group(0)
			if text[:2] == "&#":
				# character reference
				try:
					if text[:3] == "&#x":
						return unichr(int(text[3:-1], 16))
					else:
						return unichr(int(text[2:-1]))
				except ValueError:
					pass
			else:
				# named entity
				try:
					text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
				except KeyError:
					pass
			return text # leave as is
		return re.sub("&#?\w+;", fixup, text)
	def escape(self,s):
		return cgi.escape(s)

	def list_calendars(self):
		""" return a list with all calendars available with this provider"""
		return []
